import {doXHR, supportsLocalStorage, Component} from '../../utils.js';

const demoMode = (typeof sampleData != 'undefined');
const DATA_KEY_SERVERS = demoMode ? 'fylr:sample-servers' : 'fylr:saved-servers';

function escapeHTML(htmlStr) {
	return htmlStr.replace(/&/g, "&amp;")
		  .replace(/</g, "&lt;")
		  .replace(/>/g, "&gt;")
		  .replace(/"/g, "&quot;")
		  .replace(/'/g, "&#39;");
 }

function addUsernameToHost(host1, http_username, http_password) {
	let url = new URL(host1)
	try {
		let host = `${url.protocol}//`;
		if (http_username) {
			host += encodeURIComponent(http_username);
			if (http_password) {
				host += `:`+encodeURIComponent(http_password);
			}
			host += "@"
		}
		host += `${url.hostname}`;
		if (url.port) {
			host += `:${url.port}`
		}
		if (url.pathname) {
			host += url.pathname;
		}
		return host
	} catch (e) {
		console.error("error", e)
	}
	return host1

}


class ServerList extends Component {
	$formEl;
	$serverListEl;
	$serverNewEntryEl;
	$batchEl;
	$addEl;
	$delEl;
	$editEl;
	$state = {
		batchMode: false,
	};

	constructor(...args) {
		super(...args);

		// Add samples to storage
		if (demoMode) {
			this.replaceServers(sampleData);
		}

		this.$formEl = this.$('.add-form');
		this.$batchEl = this.$formEl.querySelector('form.js-batch');
		this.$addEl = this.$formEl.querySelector('.js-add-btn');
		this.$delEl = this.$formEl.querySelector('.js-delete-btn');
		this.$editEl = this.$formEl.querySelector('.js-edit-btn');
		this.$serverListEl = this.$formEl.querySelector('table.js-servers');
		this.$serverNewEntryEl = this.$serverListEl.querySelector('tr.js-add');
		this.$formEl.addEventListener('click', this.onFormClick.bind(this));
		// this.$formEl.addEventListener('input', this.onFormInput.bind(this));
		this.renderServerList();
	}

	renderServerList() {
		const servers = this.retrieveServers() || [];
		this.$serverListEl.querySelectorAll('tbody>tr:not(.js-add)').forEach((el) => {
			el.parentNode.removeChild(el);
		});
		let serverRows = servers
			.map((s, i) => ({n: i, server: s}))
			.sort((a, b) => {
				const srvA = this.parseServerEntry(a.server);
				const srvB = this.parseServerEntry(b.server);
				return (new URL(srvA.host).hostname < new URL(srvB.host).hostname) ? -1 : 1;
			});
		this.updateBatchDataElement(serverRows);
		serverRows.forEach((serverRow) => {
			const server = serverRow.server;
			const newEl = this.createServerEntryElement(serverRow.n, server);
			this.$serverNewEntryEl.parentNode.insertBefore(newEl, this.$serverNewEntryEl);
			this.updateIndicators(server, newEl);
		});
	}

	createServerEntryElement(idx, server) {
		const tr = document.createElement('tr');
		tr.setAttribute('data-id', idx);
		tr.className = 'srv-row';
		const srv = this.parseServerEntry(server);
		const srvURL = new URL(srv.host);
		const instance = srvURL.hostname.split(".")[0];
		const connectLink = this.getServerConnectLink(idx);
		tr.innerHTML = `
			<td class="indicator status"><div>&nbsp;</div></td>
			<!-- <td class="indicator auth"><div>&nbsp;</div></td> -->
			<td>${escapeHTML(instance)}</td>
			<td><a href="${srv.host}" data-action="connect">${srv.host}</a></td>
			<td>${escapeHTML(srv.login)}</td>
			<td>${escapeHTML(srv.password)}</td>
			<td>${escapeHTML(srv.http_username)}</td>
			<td>${escapeHTML(srv.http_password)}</td>
			<td class="action"><a href="${connectLink}" class="button js-connect-go" data-action="connect">Go</button></td>
		`;
		return tr;
	}

	updateIndicators(server, el) {
		const srv = this.parseServerEntry(server);
		const statusIndEl = el.querySelector('.indicator.status');
		const statusEl = statusIndEl.querySelector('div');
		// const authIndEl = el.querySelector('.indicator.auth');
		// const authEl = authIndEl.querySelector('div');
		const hostInputEl = el.querySelector('[name=server]');
		if (hostInputEl) {
			const validHost = hostInputEl.checkValidity();
			statusEl.style.display = validHost ? 'block' : 'none';
			// authEl.style.display = validHost ? 'block' : 'none';
		}
		if (!srv || !srv.host) {
			return;
		}
		this.updateServerStatus(srv)
			.then(() => {
				statusIndEl.className = 'indicator status ok';
			})
			.catch(() => {
				statusIndEl.className = 'indicator status nok';
			})
			// .finally(() => {
			// 	this.updateServerAuthStatus(srv)
			// 		.then(() => {
			// 			authIndEl.className = 'indicator auth ok';
			// 		})
			// 		.catch(() => {
			// 			authIndEl.className = 'indicator auth nok';
			// 		});
			// });
	}

	createServerEditableEntryElement(n, server) {
		const srv = this.parseServerEntry(server);
		const srvURL = new URL(srv.host);
		const tpl = this.$serverNewEntryEl.cloneNode(true);
		tpl.style.display = 'table-row';
		tpl.className = 'srv-row selected editing';
		tpl.querySelector('.instance').innerHTML = srvURL.hostname.split('.')[0];
		tpl.querySelector('[name="server"]').setAttribute('value', srv.host);
		tpl.querySelector('[name="login"]').setAttribute('value', srv.login);
		tpl.querySelector('[name="http_username"]').setAttribute('value', srv.http_username);
		tpl.querySelector('[name="http_password"]').setAttribute('value', srv.http_password);
		tpl.querySelector('[name="password"]').setAttribute('value', srv.password);
		tpl.setAttribute('data-id', n);
		const cancelEl = tpl.querySelector('[data-action="cancel"]');
		cancelEl.style.display = 'block';
		tpl.innerHTML = tpl.innerHTML
			.replace(/create/g, "update")
			.replace(/Add/g, "Update")
			.replace(/add-server/g, `update-server-${n}`);
		return tpl;
	}

	updateBatchDataElement(serverRows) {
		const textareaEl = this.$('[name="batch"]');
		textareaEl.value = serverRows.map((sr) => sr.server).join('\n');
	}

	getServerConnectLink(id) {
		const server = this.retrieveServer(id);
		const srv = this.parseServerEntry(server);
		const parts = [];
		if (srv.host) {
			let {host, username, password} = this.getHostAndBasicAuthParts(srv.host)
			const parsedUrl = new URL(host);
			let newUrl = `${parsedUrl.protocol}//`;
			try {
				username = decodeURIComponent(username)
				password = decodeURIComponent(password)
			} catch(e) {}
			if (username) {
				newUrl += username
				if (password) {
					newUrl += `:${password}`;
				}
				newUrl += '@'
			}
			newUrl += parsedUrl.host;
			if (parsedUrl.pathname) {
				newUrl += parsedUrl.pathname;
			}
			parts.push(`server=${encodeURIComponent(newUrl)}`);
		}
		if (srv.login) {
			parts.push(`login=${encodeURIComponent(srv.login)}`);
		}
		if (srv.password) {
			parts.push(`password=${encodeURIComponent(srv.password)}`);
		}
		return `/?${parts.join('&')}`;
	}

	onFormClick(evt) {
		const actionEl = evt.target;
		const action = actionEl.dataset.action;
		if (action === "connect") {
			return
		}
		evt.preventDefault();
		switch (action) {
			case "create":
				if (actionEl.form.reportValidity()) {
					this.createServerEntry(this.getFormData(actionEl.form)[0]);
					const srvEl = actionEl.closest('tr');
					srvEl.parentNode.removeChild(srvEl);
				}
				break;
			case "update":
				if (actionEl.form.reportValidity()) {
					const srvRow = actionEl.closest('.srv-row');
					const id = srvRow.dataset.id;
					this.updateServerEntry(parseInt(id, 10), this.getFormData(actionEl.form)[0]);
					const el = this.$serverListEl.querySelector(`.srv-row[data-id="${id}"]`);
					el.className = srvRow.className.replace('editing', '');
					this.checkSelectedButtons();
				}
				break;
			case "add":
				this.createAddServerEl();
				break;
			case "delete":
				const el = this.$serverListEl.querySelector('.srv-row.selected');
				const id = el.dataset && el.dataset.id;
				if (!id) {
					el.parentNode.removeChild(el);
				}
				this.removeServerEntry(parseInt(id, 10));
				return this.checkDeleteButton();
			case "apply":
				if (this.$batchEl.reportValidity()) {
					this.replaceServerList(this.getFormData(this.$batchEl));
					this.$state.batchMode = false;
					this.switchBatchEdit();
					this.$addEl.style.display = 'block';
				}
				break;
			case "edit":
				return this.switchToEdit();
			case "cancel":
				return this.cancelEdit(actionEl);
			case "batch":
				this.$addEl.style.display = 'none';
				this.$state.batchMode = true
				this.switchBatchEdit();
				return this.checkSelectedButtons()
			default:
				const tr = actionEl.parentNode;
				if (tr.className.indexOf('srv-row') === -1) {
					return;
				}
				const selected = tr.className.indexOf('selected') !== -1;
				const selClass = selected ? '' : 'selected';
				const allSrvRows = this.$serverListEl.querySelectorAll('.srv-row');
				allSrvRows.forEach((sr) => sr.className = sr.className.replace(/selected/, ''));
				tr.className += ` ${selClass}`;
				this.checkSelectedButtons();
				return;
		}
	}

	onFormInput(evt) {
		this.onEntryChange(evt.target.form);
	}

	onEntryChange(formEl) {
		if (!formEl) {
			return;
		}
		const entryEl = formEl.closest('tr');
		if (!entryEl) {
			return;
		}
		const server = this.getFormData(formEl)[0];
		const srv = this.parseServerEntry(server);
		if (srv && srv.host) {
			entryEl.querySelector('[name=server]').value = srv.host;
			entryEl.querySelector('[name=login]').value = srv.login || '';
			entryEl.querySelector('[name=password]').value = srv.password || '';
		}
		this.updateIndicators(server, entryEl);
	}

	cancelEdit(actionEl, action) {
		const oldEl = actionEl.closest('.srv-row');
		if (oldEl) {
			const id = parseInt(oldEl.dataset.id, 10);
			const server = this.retrieveServer(id);
			const newEl = this.createServerEntryElement(id, server);
			newEl.className = oldEl.className.replace('editing', '');
			oldEl.replaceWith(newEl);
			this.checkSelectedButtons();
			this.updateIndicators(server, newEl);
		}
	}

	switchToEdit() {
		const oldEl = this.$serverListEl.querySelector('.srv-row.selected');
		if (oldEl) {
			const id = parseInt(oldEl.dataset.id, 10);
			const server = this.retrieveServer(id);
			oldEl.replaceWith(this.createServerEditableEntryElement(id, server));
			this.$editEl.style.display = 'none';
		}
	}

	createServerEntry(server) {
		this.addServer(server);
		this.renderServerList();
		this.$serverNewEntryEl.querySelector('[name=server]').value = "";
		this.$serverNewEntryEl.querySelector('[name=login]').value = "";
		this.$serverNewEntryEl.querySelector('[name=password]').value = "";
		this.$serverNewEntryEl.querySelector('[name=http_username]').value = "";
		this.$serverNewEntryEl.querySelector('[name=http_password]').value = "";
	}

	createAddServerEl() {
		const now = new Date().getTime();
		const tpl = this.$serverNewEntryEl.cloneNode(true);
		tpl.className = "js-add srv-row";
		tpl.innerHTML = tpl.innerHTML
			.replace(/add-server/g, `add-server-${now}`);
		tpl.style.display = 'table-row';
		this.$serverListEl.appendChild(tpl);
	}

	checkSelectedButtons() {
		const selEl = this.$serverListEl.querySelector('.srv-row.selected');
		this.$delEl.style.display = (selEl && !this.$state.batchMode) ? 'block' : 'none';
		const editing = selEl && selEl.className.indexOf('editing') !== -1;
		this.$editEl.style.display = (selEl && !editing && !this.$state.batchMode) ? 'block' : 'none';
	}

	updateServerEntry(id, server) {
		this.updateServer(id, server);
		this.renderServerList();
	}

	removeServerEntry(id) {
		this.removeServerEntries([id]);
	}

	removeServerEntries(ids) {
		if (!ids || ids.length < 1) {
			return;
		}
		this.removeServers(ids);
		this.renderServerList();
	}

	replaceServerList(servers) {
		this.replaceServers(servers);
		this.renderServerList();
	}

	switchBatchEdit() {
		this.$batchEl.style.display = this.$state.batchMode ? 'block' : 'none';
		this.$serverListEl.style.display = this.$state.batchMode ? 'none' : 'table';
		const btnEl = this.$formEl.querySelector('button.js-batch');
		btnEl.innerHTML = this.$state.batchMode ? 'Done' : 'Batch';
		btnEl.setAttribute('data-action', this.$state.batchMode ? 'apply' : 'batch');
	}

	getFormData(form) {
		let servers = []
		if (this.$state.batchMode) {
			const batchValue = form.elements.batch?.value
			servers = this.parseBatchData(batchValue)
		} else {
			let host = form.elements.server?.value
			const srv = this.parseServerEntry(host)
			if (!srv) {
				console.error("Could not parse host url")
				return []
			}
			const login = encodeURIComponent(form.elements.login?.value || srv.login)
			const password = encodeURIComponent(form.elements.password?.value || srv.password)
			const http_username = form.elements.http_username?.value || srv.http_username
			const http_password = form.elements.http_password?.value || srv.http_password
			if (srv.host) {
				host = addUsernameToHost(srv.host, http_username, http_password)
				servers.push(`${host}?login=${login}&password=${password}`)
			}
		}
		return servers
	}

	parseBatchData(value) {
		return value.split("\n").map((line) => line.trim()).filter(Boolean);
	}

	parseServerEntry(entry) {
		// This will fail if data is malformed
		// We wipe the local storage in this case
		try {
			entry = (entry || '').trim();
		} catch(e) {
			console.error(`Data corrupted: ${e}. Wiping local storage`);
			this.clearServers();
			entry = '';
		}
		if (!entry) {
			return {};
		}
		let url;
		try {
			url = new URL(entry);
		} catch(e) {
			return {};
		}
		const login = url.searchParams.has("login") ? url.searchParams.get("login") : '';
		const password = url.searchParams.has("password") ? url.searchParams.get("password") : '';
		const http_username = decodeURIComponent(url.username);
		const http_password = decodeURIComponent(url.password);
		const server = url.searchParams.get("server");
		let host = server;
		if (!host) {
			// build host from parse url
			try {
				host = url.protocol+"//"+url.hostname;
				if (url.port) {
					host += `:${url.port}`
				}
				if (url.pathname) {
					host += url.pathname;
				}
			} catch(e) {
				console.error(e);
			}
		}
		return {host, login, password, http_username, http_password};
	}

	addServer(server) {
		if (supportsLocalStorage()) {
			const servers = this.retrieveServers();
			servers.push(server);
			localStorage.setItem(DATA_KEY_SERVERS, JSON.stringify(servers));
		}
	}

	updateServer(id, server) {
		if (supportsLocalStorage()) {
			const servers = this.retrieveServers();
			servers[id] = server;
			localStorage.setItem(DATA_KEY_SERVERS, JSON.stringify(servers));
		}
	}

	replaceServers(servers) {
		if (supportsLocalStorage()) {
			localStorage.setItem(DATA_KEY_SERVERS, JSON.stringify(servers));
		}
	}

	removeServers(ids) {
		if (supportsLocalStorage()) {
			let servers = this.retrieveServers();
			servers = servers.filter((srv, idx) => {
				const ok = ids.indexOf(idx) === -1;
				return ok
			});
			localStorage.setItem(DATA_KEY_SERVERS, JSON.stringify(servers));
		}
	}

	clearServers() {
		if (supportsLocalStorage()) {
			localStorage.removeItem(DATA_KEY_SERVERS);
		}
	}

	retrieveServer(id) {
		if (supportsLocalStorage()) {
			const servers = this.retrieveServers();
			return servers[id];
		}
	}

	retrieveServers() {
		let servers = [];
		if (supportsLocalStorage()) {
			servers = JSON.parse(localStorage.getItem(DATA_KEY_SERVERS) || '[]');
		}
		return servers;
	}

	async updateServerStatus(srv) {
		const {host, headers} = this.getHostAndHeaders(srv.host.replace(/\/$/, ''));
		const url = `${host}api/v1/settings`;
		return new Promise(async (resolve, reject) => {
			try {
				srv.info = await fetch(url, {headers}).then(response => response.json());
				resolve({server: srv});
			} catch(e)  {
				srv.info = null;
				reject({error: e, server: srv});
			}
		});
	}

	async updateServerAuthStatus(srv) {
		const {host, headers} = this.getHostAndHeaders(srv.host.replace(/\/$/, ''));
		const url = `${host}/api/v1/session/authenticate`;
		const formParts = [];
		formParts.push(`login=${encodeURIComponent(srv.login)}`);
		formParts.push(`password=${encodeURIComponent(srv.password)}`);
		return new Promise(async (resolve, reject) => {
			try {
				let xhr = await fetch(`${host}/api/v1/session`, {headers});
				const res = await xhr.json();
				formParts.push(`access_token=${encodeURIComponent(res.token)}`);
				headers.append('Content-Type', 'application/x-www-form-urlencoded')
				const formData = formParts.join('&').replace(/%20/g, '+');
				xhr = await fetch(url, {
					method: 'POST',
					mode: 'cors',
					credentials: 'include',
					body: formData,
					headers: headers
				});
				if (xhr.status !== 200) {
					reject({error: `Received status ${xhr.status}: ${xhr.statusText}`, server: srv})
				}
				srv.auth = {token: res.token};
				resolve({server: srv});
			} catch (e) {
				srv.auth = null;
				reject({error: e, server: srv});
			}
		});
	}

	getHostAndBasicAuthParts(rawUri) {
		const uri = new URL(rawUri);
		let host = `${uri.protocol}//${uri.host}`;
		if (uri.pathname) {
			host += uri.pathname;
		}
		// The parsed auth part does not work with some chars like $
		const parts = rawUri.split('@');
		let username = '';
		let password = '';
		if (parts.length > 1) {
			const authParts = parts[0].slice(uri.protocol.length + 2).split(':');
			username = authParts[0];
			password = authParts[1];
		}
		return {host, username, password};
	}

	getHostAndHeaders(rawUri) {
		const headers = new Headers();
		const uri = new URL(rawUri);
		let bAuth = '';
		let {host, username, password} = this.getHostAndBasicAuthParts(rawUri);
		if (username) {
			try {
				username = decodeURIComponent(username);
			} catch (e) {}
			bAuth += username;
		}
		if (password) {
			try {
				password = decodeURIComponent(password);
			} catch (e) {}
			bAuth += `:${password}`;
		}
		if (bAuth) {
			headers.append('Authorization', 'Basic ' + btoa(bAuth));
		}
		return {host, headers};
	}
}


document.addEventListener('DOMContentLoaded', (evt) => {
	new ServerList('.page-web-only .server-section');
});
